"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
}
Object.defineProperty(exports, "__esModule", { value: true });
var _ = __importStar(require("lodash"));
var unix_index_map_1 = require("./unix-index-map");
var EnumBrowseDirection;
(function (EnumBrowseDirection) {
    EnumBrowseDirection[EnumBrowseDirection["Backward"] = 0] = "Backward";
    EnumBrowseDirection[EnumBrowseDirection["Forward"] = 1] = "Forward";
})(EnumBrowseDirection || (EnumBrowseDirection = {}));
var getBucketOutside = function (allData, browseDirection, filterFrom, filterTo) {
    var sample = browseDirection == EnumBrowseDirection.Backward ?
        _.last(_.filter(allData, function (el) { return el.unix <= filterFrom; })) :
        _.first(_.filter(allData, function (el) { return el.unix >= filterTo; }));
    return _.isObject(sample) ?
        {
            minY: sample.value,
            maxY: sample.value,
            date: new Date(sample.unix),
            numberOfSamples: 1,
            firstSample: sample,
            lastSample: sample,
            unixFrom: sample.unix,
            unixTo: sample.unix
        } : null;
};
var getTimeSeriesBuckets = function (allData, unixToIndexMap, numberOfBuckets, filterFrom, filterTo) {
    if (allData.length == 0)
        return {
            buckets: [],
            shadowPreceding: null,
            shadowSucceeding: null
        };
    filterFrom = _.isNumber(filterFrom) ? filterFrom : _.first(allData).unix;
    filterTo = _.isNumber(filterTo) ? filterTo : _.last(allData).unix;
    var bucketLengthUnix = (filterTo - filterFrom) / numberOfBuckets;
    var indexFrom = unix_index_map_1.unixIndexMapCalculations.findIndexToBrowsePointsFrom(allData, unixToIndexMap, filterFrom);
    var buckets = [];
    var referenceBucket = null;
    for (var i = indexFrom; i < allData.length; i++) {
        var el = allData[i];
        if (el.unix > filterTo) {
            break;
        }
        if ((referenceBucket == null) || (referenceBucket.unixTo < el.unix)) {
            var bucketIndex = _.floor((el.unix - filterFrom) / bucketLengthUnix);
            referenceBucket = {
                unixFrom: filterFrom + bucketIndex * bucketLengthUnix,
                unixTo: filterFrom + (bucketIndex + 1) * bucketLengthUnix,
                firstSample: el,
                lastSample: el,
                date: new Date(filterFrom + (bucketIndex + 0.5) * bucketLengthUnix),
                numberOfSamples: 0,
                minY: el.value,
                maxY: el.value,
            };
            buckets.push(referenceBucket);
        }
        referenceBucket.lastSample = el;
        referenceBucket.numberOfSamples++;
        referenceBucket.minY = el.value < referenceBucket.minY ? el.value : referenceBucket.minY;
        referenceBucket.maxY = el.value > referenceBucket.maxY ? el.value : referenceBucket.maxY;
    }
    return {
        buckets: _.filter(buckets, function (b) { return b.unixFrom >= filterFrom && b.unixTo <= filterTo; }),
        shadowPreceding: getBucketOutside(allData, EnumBrowseDirection.Backward, filterFrom, filterTo),
        shadowSucceeding: getBucketOutside(allData, EnumBrowseDirection.Forward, filterFrom, filterTo)
    };
};
var convertToOnScreenTimeSeries = function (series, unixFrom, unixTo, numberOfBuckets) {
    var grouped = _.toArray(_.groupBy(series, function (ts) { return ts.name; }));
    return _.reduce(grouped, function (acc, col) {
        return _.concat(acc, _.map(col, function (ts) {
            return ({
                name: ts.name,
                fragmentId: _.indexOf(col, ts),
                color: ts.color,
                buckets: getTimeSeriesBuckets(ts.points, ts.unixToIndexMap, numberOfBuckets, unixFrom, unixTo),
                type: ts.type
            });
        }));
    }, []);
};
exports.hpTimeSeriesChartCalculations = {
    getTimeSeriesBuckets: getTimeSeriesBuckets,
    convertToOnScreenTimeSeries: convertToOnScreenTimeSeries,
};
//# sourceMappingURL=index.js.map